using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using Microsoft.Surface;
using Microsoft.Surface.Core;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Collections.Generic;
using System.Collections;
//using GroupLab.Networking;
//using GroupLab.SharedPhidgets;
//using GroupLab.SharedPhidgets.Utilities;
using System.IO;
using System.Windows.Shapes;
using Petzold.Media2D;
using System.Windows.Interop;
using System.Timers;


namespace TTT
{
    /// <summary>
    /// TTT - Tactile TableTop Project
    /// </summary>
    public partial class MainWindow
    {

        #region Members and constant members

        /// <summary>
        /// Layers for image, material, etc.
        /// Using "Layers" enumeration for identification.
        /// </summary>
        private Dictionary<string, System.Drawing.Bitmap> layers = new Dictionary<string, System.Drawing.Bitmap>();

        /// <summary>
        /// Interval of the check position timer
        /// </summary>
        public static int POSITION_TIMER_INTERVAL_IN_MILLISECONDS = 50;

        /// <summary>
        /// Contact observers
        /// </summary>
        private Dictionary<string, TTTArtefact> contactObservers = new Dictionary<string, TTTArtefact>();


        /// <summary>
        /// Contact targets   
        /// </summary>
        private ContactTarget contactTarget;

        /// <summary>
        /// Layer names
        /// </summary>
        public static string LAYER_VISIBLEIMAGE     = "VisibleImage";
        public static string LAYER_RELIEF           = "Relief";
        public static string LAYER_HAPTICRESPONSE   = "HapticResponse";
        public static string LAYER_OSCILLATION      = "Oscillation";
        public static string LAYER_AMPLITUDE        = "Amplitude";
        public static string LAYER_PHASE            = "Phase";
        public static string LAYER_BRAKE            = "Brake";
        public static string LAYER_RATIO            = "Ratio";


        /// <summary>
        /// The default directory for image masks
        /// </summary>
        public static string DEFAULT_DIRECTORY = @"C:\Development\Masks\";

        /// <summary>
        /// The default shared dictionary address
        /// </summary>
        public static string DEFAULT_ADDRESS = "tcp://localhost:sp";

        /// <summary>
        /// Screen width
        /// </summary>
        private static int SCREEN_WIDTH = 1024;

        /// <summary>
        /// Screen height
        /// </summary>
        private static int SCREEN_HEIGHT = 768;

        /// <summary>
        /// Shared dictionary connection (for shared phidgets and access to control app)
        /// </summary>
        //private SharedDictionary sharedDictionary = null;
        
        /// <summary>
        /// Subscribing to dictionary entries 
        /// </summary>
        //private Subscription subscription = null;

        /// <summary>
        /// Connection manager for opening shared phidgets connection
        /// </summary>
        //private ConnectionManager connectionManager = null;

        /// <summary>
        /// Layer identifiers
        /// </summary>
        public List<string> LayerIdentifier = new List<string>();


        private DispatcherTimer timerCheckPositions = null;

        private List<string> markerCheck = new List<string>();

        private DemonstrationPreset currentPreset = DemonstrationPreset.Default;


        private Dictionary<string, TTTArtefact> tttArtefacts = new Dictionary<string, TTTArtefact>();

        private RadialGradientBrush radialGradientBrush;

        private SolidColorBrush solidBrushGreen;
        private SolidColorBrush solidBrushYellow;


        private Dictionary<string, Rect> containers = new Dictionary<string, Rect>();
        private Dictionary<string, Rect> validDrawingRegions = new Dictionary<string, Rect>();
        private Dictionary<string, string> draggedObjects = new Dictionary<string, string>();

        #endregion



        #region Delegates

        delegate void InitializeLayersDelegate(string directoryInt, string identifierInt, string extensionInt);

        #endregion



        #region Constructor

        /// <summary>
        /// Default constructor.
        /// </summary>
        public MainWindow()
        {
            LayerIdentifier.Add(LAYER_VISIBLEIMAGE);
            LayerIdentifier.Add(LAYER_RELIEF);
            LayerIdentifier.Add(LAYER_HAPTICRESPONSE);
            LayerIdentifier.Add(LAYER_OSCILLATION);
            LayerIdentifier.Add(LAYER_AMPLITUDE);
            LayerIdentifier.Add(LAYER_PHASE);
            LayerIdentifier.Add(LAYER_BRAKE);
            LayerIdentifier.Add(LAYER_RATIO); 
 
            InitializeComponent();

            // Add handlers for Application activation events
            AddActivationHandlers();

            this.contactVisualizer.SetParent(this);           
            
            this.InitializeLayers(DEFAULT_DIRECTORY, "Photo 1", "png");

            //this.InitializeArtefacts();


            /*this.connectionManager = new ConnectionManager();
            this.connectionManager.SharedDictionaryURL = DEFAULT_ADDRESS;
            this.sharedDictionary = this.connectionManager.GetSharedDictionary();
            this.connectionManager.OpenConnection();*/

            _handle = new TTT.Utilities.LazyLoader<IntPtr>(delegate { return new WindowInteropHelper(this).Handle; });



            // Create a target for surface input.
            contactTarget = new ContactTarget(this.Handle, EventThreadChoice.OnBackgroundThread);
            contactTarget.EnableInput();

            this.InitializePositionTimer();

            this.InitializeContainers(this.currentPreset);

            this.InitializeValidDrawingRegions();

            this.LoadFilesFromDirectory();

            this.InitializeBrushes();

            this.InitPentimento();

            controlWindow = new ControlWindow(this);
            controlWindow.Show();

            this.InitializePhidgets();

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();

        }

        private ControlWindow controlWindow;
        private DispatcherTimer timer;


        void timer_Tick(object sender, EventArgs e)
        {
            if (this.isServoAttached && this.isInterfaceKitAttached)
            {
                this.timer.Stop();
                this.controlWindow.PhidgetsAttached();
                this.InitializeArtefacts();              
            }
        }

        /*public Dictionary<string, TTTArtefact> TTTArtefacts
        {
            get
            {
                return this.tttArtefacts;
            }
        }*/

        public void SetManualArtefactControl(bool isManualControl)
        {
            foreach (TTTArtefact artefact in this.tttArtefacts.Values)
            {
                artefact.ManualControl = isManualControl;
            }
        }

        public Phidgets.AdvancedServo AdvancedServoController
        {
            get
            {
                return this.advancedServoController;
            }
        }



        private void InitializeBrushes()
        {
            this.radialGradientBrush = new RadialGradientBrush();
            this.radialGradientBrush.Center = new Point(0.5, 0.5);
            this.radialGradientBrush.GradientOrigin = new Point(0.5, 0.5);
            this.radialGradientBrush.RadiusX = 0.5;
            this.radialGradientBrush.RadiusY = 0.5;

            this.radialGradientBrush.GradientStops.Add(
                new GradientStop(Colors.Blue, 0.6));
            this.radialGradientBrush.GradientStops.Add(
                new GradientStop(Color.FromArgb(0, 255,255,255), 1.0));

            this.solidBrushGreen = new SolidColorBrush(Colors.Green);
            this.solidBrushYellow = new SolidColorBrush(Colors.Yellow);
        }


        private void SetDemonstrationPreset(DemonstrationPreset preset)
        {
            this.currentPreset = preset;

            foreach (TTTArtefact artefact in this.tttArtefacts.Values)
            {
                artefact.SetDemonstrationPreset(preset);
            }
        }


        private void LoadFilesFromDirectory()
        {
            this.surfaceListBoxFiles.Items.Clear();
            foreach (string file in Directory.GetFiles(DEFAULT_DIRECTORY, "*.png"))
            {
                if (System.IO.Path.GetFileNameWithoutExtension(file).ToLower().EndsWith("visibleimage"))
                {
                    this.surfaceListBoxFiles.Items.Add(System.IO.Path.GetFileNameWithoutExtension(file).Replace("_VisibleImage", ""));
                }
            }

        }


        public bool IsDraggedObject(string objectID)
        {
            if (this.draggedObjects.ContainsValue(objectID)) return true;
            else return false;
        }

        public void SetDraggedObject(string artefactID, string objectID)
        {
            foreach (string key in this.draggedObjects.Keys)
            {
                if (this.draggedObjects[key] == objectID)
                {
                    this.tttArtefacts[key].Override();

                    this.draggedObjects.Remove(key);
                    this.draggedObjects.Add(artefactID, objectID);
                }
                else
                {
                    this.draggedObjects.Add(artefactID, objectID);
                }
            }
        }

        public void UnsetDraggedObject(string artefactID)
        {
            this.draggedObjects.Remove(artefactID);
        }




        private void InitializeContainers(DemonstrationPreset preset)
        {
            this.containers.Clear();


            if (preset == DemonstrationPreset.FloorPlanning)
            {
                this.AddContainer("Brake", new Rect(0,0,182, 768));
                this.AddContainer("Room1", new Rect(187,526,127,178));
                this.AddContainer("Room2", new Rect(325, 527, 130, 177));
                this.AddContainer("Room3", new Rect(462, 529, 130, 176));
                this.AddContainer("Room4", new Rect(583, 131, 282, 262));
            }
            else if (preset == DemonstrationPreset.Art)
            {
                this.AddContainer("Man in painting", new Rect(650,220, 97, 192));
                this.AddContainer("Door to another room", new Rect(230, 150, 111, 222));
                this.AddContainer("Painting on wall", new Rect(760, 60, 109, 215));
            }
            else if (preset == DemonstrationPreset.Payment)
            {
                this.AddContainer("Button", new Rect(672, 587, 202, 62));
            }
            else
            {
                //Geometry g = this.rectangleInner.RenderedGeometry.Bounds
                this.AddContainer(this.boxInner.Name, this.boxInner);
                this.AddContainer(this.boxOuter.Name, this.boxOuter);
            }
            
        }


        private void InitializeValidDrawingRegions()
        {
            this.validDrawingRegions.Clear();

            this.validDrawingRegions.Add("1", new Rect(0, 0, 128, 768));
            this.validDrawingRegions.Add("2", new Rect(256, 0, 128, 768));
            this.validDrawingRegions.Add("3", new Rect(512, 0, 128, 768));
            this.validDrawingRegions.Add("4", new Rect(768, 0, 128, 768));
        }


        public bool IsValidPaintingPoint(Point point)
        {
            // Override this function
            return true;

            foreach (string key in this.validDrawingRegions.Keys)
            {
                if (this.validDrawingRegions[key].Contains(point))
                {
                    return true;
                }
            }

            return false;
        }


        private Dictionary<string, InfoBox> infoBoxes = new Dictionary<string, InfoBox>();



        public void HideInfoDialog(string htp)
        {
            if (infoBoxes.ContainsKey(htp))
            {
                ((InfoBox)this.infoBoxes[htp]).Visibility = Visibility.Hidden;
            }
        }



        public void ShowInfoDialog(string id, string htp, Point location, double angle)
        {
            double width = 277;
            double height = 78;


            if (infoBoxes.ContainsKey(htp))
            {
                InfoBox box = (InfoBox)this.infoBoxes[htp];

                box.SetValue(Canvas.LeftProperty, location.X);
                box.SetValue(Canvas.TopProperty, location.Y);

                box.SetValue(Canvas.WidthProperty, width);
                box.SetValue(Canvas.HeightProperty, height);

                box.SetText(id, "");
                box.Visibility = Visibility.Visible;

                ((RotateTransform)box.RenderTransform).Angle = Utilities.RadianToDegree(angle) + 90.0;
            }
            else
            {
                InfoBox box = new InfoBox();
                box.RenderTransform = new RotateTransform(Utilities.RadianToDegree(angle) + 90.0);

                box.SetText(id, "");

                box.SetValue(Canvas.LeftProperty, location.X );
                box.SetValue(Canvas.TopProperty, location.Y);

                box.SetValue(Canvas.WidthProperty, width);
                box.SetValue(Canvas.HeightProperty, height);

                this.canvasPointers.Children.Add(box);
                this.infoBoxes.Add(htp, box);
            }
        }




        public bool MoveContent(string HTPid, string contentID, Point newPoint, int pressureLayer)
        {
            return true;
        }



        public bool IsContainer(string name)
        {
            if (this.containers.ContainsKey(name)) return true;
            else return false;
        }


        private void AddContainer(string ID, UIElement container)
        {
            this.containers.Add(ID, new Rect(
                (double)container.GetValue(Canvas.LeftProperty),
                (double)container.GetValue(Canvas.TopProperty),
                (double)container.GetValue(Canvas.WidthProperty),
                (double)container.GetValue(Canvas.HeightProperty)));
        }

        private void AddContainer(string ID, Rect rect)
        {
            this.containers.Add(ID, rect);          
        }


        public string GetContainerIntersection(Point pt)
        {
            foreach (string key in this.containers.Keys)
            {
                if (this.containers[key].Contains(pt))
                {
                    return key;
                }
            }

            return "none";
        }

        public bool IsInContainer(Rect rect)
        {
            foreach (string key in this.containers.Keys)
            {
                if ((this.containers[key].Contains(rect.Location)) && (this.containers[key].Contains(rect.BottomRight)))
                {
                    return true;
                }
            }

            return false;
        }

        public bool IsOutsideOfContainer(string container, Rect rect)
        {
            if ((this.containers[container].Contains(rect.Location)) && (this.containers[container].Contains(rect.BottomRight)))
            {
                return false;
            }
            return true;
        }



        public string GetContainerIntersection(Rect rect)
        {
            foreach (string key in this.containers.Keys)
            {
                if ((this.containers[key].Contains(rect.Location)) && (this.containers[key].Contains(rect.BottomRight)))
                {
                    return key;
                }
            }

            return "";
        }
       

        #endregion



        #region Check Position Timer

        private void InitializePositionTimer()
        {
            this.timerCheckPositions = new DispatcherTimer();
            this.timerCheckPositions.Interval = new System.TimeSpan(0, 0, 0, 0, POSITION_TIMER_INTERVAL_IN_MILLISECONDS);
            this.timerCheckPositions.Tick += new EventHandler(timerCheckPositions_Tick);
            this.timerCheckPositions.Start();

        }


        void timerCheckPositions_Tick(object sender, EventArgs e)
        {          
            ReadOnlyContactCollection contacts = contactTarget.GetState();

            this.markerCheck = new List<string>(this.contactObservers.Keys);

            foreach (Contact contact in contacts)
            {
                if (contact.Tag.Type == TagType.Byte)
                {
                    string id = contact.Tag.Byte.Value.ToString();

                    if (this.contactObservers.ContainsKey(id))
                    {
                        ((TTTArtefact)this.contactObservers[id]).UpdatePosition(1024 - Math.Abs(contact.X), Math.Abs(contact.Y), contact.Orientation);
                        
                        this.markerCheck.Remove(id);
                    }
                }
            }

            foreach(string id in this.markerCheck)
            {
                ((TTTArtefact)this.contactObservers[id]).RemoveContact();
            }

            
        }

        internal void RegisterCallback(string markerID, TTTArtefact artefact)
        {
            this.contactObservers.Add(markerID, artefact);

        }

        internal void UnregisterCallback(string markerID, TTTArtefact artefact)
        {
            this.contactObservers.Remove(markerID);

        }

        #endregion



        #region Window Handle
        private TTT.Utilities.LazyLoader<IntPtr> _handle;
        protected IntPtr Handle
        {
            get
            {
                return _handle.Value;
            }
        }
        #endregion





        #region Initialize Phidgets Hardware

        private Phidgets.AdvancedServo advancedServoController = null;
        private Phidgets.InterfaceKit interfaceKit = null;

        private bool isServoAttached = false;
        private bool isInterfaceKitAttached = false;

        private void InitializePhidgets()
        {
            this.advancedServoController = new Phidgets.AdvancedServo();
            this.advancedServoController.Attach += new Phidgets.Events.AttachEventHandler(advancedServoController_Attach);
            //this.servoHeight.FilterSerialNumbers.Add(servoHeightSerial);

            //int serial = int.Parse(servoHeightSerial);
            this.advancedServoController.open(88419);


            this.interfaceKit = new Phidgets.InterfaceKit();
            this.interfaceKit.Attach += new Phidgets.Events.AttachEventHandler(interfaceKit_Attach);
            this.interfaceKit.open(68103);

        }

        public Phidgets.InterfaceKit InterfaceKit
        {
            get
            {
                return this.interfaceKit;
            }
        }


        public Phidgets.AdvancedServoServo GetServoMotor(int index)
        {
            if (this.isServoAttached && this.advancedServoController != null)
            {
                return this.advancedServoController.servos[index];
            }

            return null;
        }


        void interfaceKit_Attach(object sender, Phidgets.Events.AttachEventArgs e)
        {
            isInterfaceKitAttached = true;
        }



        void advancedServoController_Attach(object sender, Phidgets.Events.AttachEventArgs e)
        {
            Phidgets.AdvancedServo attached = (Phidgets.AdvancedServo)sender;


            try
            {
                //Set the default servo type to the one Phidgets sells
                foreach (Phidgets.AdvancedServoServo motor in attached.servos)
                {

                    motor.Type = Phidgets.ServoServo.ServoType.HITEC_HS322HD;
                    motor.Engaged = true;
                    motor.SpeedRamping = false;
                }
            }
            catch (Phidgets.PhidgetException ex) { }

       


            isServoAttached = true;
        }



        #endregion



        #region Initialize TTT devices


        /// <summary>
        /// Initialize the TTT devices.
        /// </summary>
        private void InitializeArtefacts()
        {
            /*TTTArtefact artefact1 = new TTTArtefact(
                "258",                   // Servo height serial
                0,                       // Servo height port 
                178,                     // Servo height maximum 
                136,                     // Servo height minimum
                ((byte)0xE7).ToString(), // Bytetag
                "68103",                 // Interfacekit serial
                2,                       // Sensor port
                this,                    // Parent
                Colors.DarkGreen,        // Paint color
                Colors.Green);           // Arrow color
            this.tttArtefacts.Add(artefact1.GetArtefactID(), artefact1);


            TTTArtefact artefact2 = new TTTArtefact(
                "74", 
                0, 168, 123, 
                ((byte)0xF4).ToString(), 
                "68103", 
                1, 
                this, 
                Colors.DarkRed,
                Colors.Red);
            this.tttArtefacts.Add(artefact2.GetArtefactID(), artefact2);
            artefact2.InitializeBreak("407", 0, 168, 173);
            


            TTTArtefact artefact3 = new TTTArtefact(
                "392",
                0, 139, 70,
                ((byte)0xA2).ToString(),
                "68103",
                2,
                this,
                Colors.DarkRed,
                Colors.Orange);
            this.tttArtefacts.Add(artefact3.GetArtefactID(), artefact3);
            artefact3.SetMaximumSensorValue(554.0);
            artefact3.SetPointerArrow(80.0, 0.0);
            */
            


            // New complete HTP
            TTTArtefact artefact4 = new TTTArtefact(
                "88419",
                0, 
                120, 
                26,
                ((byte)0xCE).ToString(),
                "68103",
                0,
                this,
                Colors.DarkRed,
                Colors.Red);

            this.tttArtefacts.Add(artefact4.GetArtefactID(), artefact4);
            artefact4.SetMaximumSensorValue(800.0);
            artefact4.SetPointerArrow(72.0, 0.0);
            artefact4.InitializeBreak("88419", 1, 180, 167);



            TTTArtefact artefact5 = new TTTArtefact(
                "88419",                   // Servo height serial
                2,                       // Servo height port 
                110,                     // Servo height maximum 
                50,                     // Servo height minimum
                ((byte)0x1A).ToString(), // Bytetag
                "68103",                 // Interfacekit serial
                1,                       // Sensor port
                this,                    // Parent
                Colors.DarkGreen,        // Paint color
                Colors.Green);           // Arrow color
            this.tttArtefacts.Add(artefact5.GetArtefactID(), artefact5);
            artefact5.SetMaximumSensorValue(554.0);
            artefact5.SetPointerArrow(80.0, 0.0);

            TTTArtefact artefact6 = new TTTArtefact(
                "88419",                   // Servo height serial
                3,                       // Servo height port 
                66,                     // Servo height maximum 
                2,                     // Servo height minimum
                ((byte)0x5E).ToString(), // Bytetag
                "68103",                 // Interfacekit serial
                2,                       // Sensor port
                this,                    // Parent
                Colors.DarkBlue,        // Paint color
                Colors.Blue);           // Arrow color
            this.tttArtefacts.Add(artefact6.GetArtefactID(), artefact6);
            artefact6.SetMaximumSensorValue(554.0);
            artefact6.SetPointerArrow(80.0, 0.0);


            TTTArtefact artefact7 = new TTTArtefact(
                "88419",                   // Servo height serial
                4,                       // Servo height port 
                92,                     // Servo height maximum 
                38,                     // Servo height minimum
                ((byte)0xEF).ToString(), // Bytetag
                "68103",                 // Interfacekit serial
                3,                       // Sensor port
                this,                    // Parent
                Colors.DarkOrange,        // Paint color
                Colors.Orange);           // Arrow color
            this.tttArtefacts.Add(artefact7.GetArtefactID(), artefact7);
            artefact7.SetMaximumSensorValue(554.0);
            artefact7.SetPointerArrow(80.0, 0.0);



            //artefact1.StartHapticLoop();
            //artefact2.StartHapticLoop();
            //artefact3.StartHapticLoop();
            artefact4.StartHapticLoop();
            artefact5.StartHapticLoop();
            artefact6.StartHapticLoop();
            artefact7.StartHapticLoop();
        }



        public void InitializeOldHTPs()
        {
             TTTArtefact artefact1 = new TTTArtefact(
                "88419",                   // Servo height serial
                7,                       // Servo height port 
                155,                     // Servo height maximum 
                102,                     // Servo height minimum
                ((byte)0xE7).ToString(), // Bytetag
                "68103",                 // Interfacekit serial
                4,                       // Sensor port
                this,                    // Parent
                Colors.DarkGreen,        // Paint color
                Colors.Green);           // Arrow color
            this.tttArtefacts.Add(artefact1.GetArtefactID(), artefact1);


            TTTArtefact artefact2 = new TTTArtefact(
                "88419", 
                6, 
                150, 
                87, 
                ((byte)0xF4).ToString(), 
                "68103", 
                3, 
                this, 
                Colors.DarkRed,
                Colors.Red);
            this.tttArtefacts.Add(artefact2.GetArtefactID(), artefact2);
            artefact2.InitializeBreak("88419", 5, 138, 147);

            artefact1.StartHapticLoop();
            artefact2.StartHapticLoop();
        }



        #endregion



        #region Shared dictionary handler


/*
        void subscription_Notified(object sender, SubscriptionEventArgs e)
        {
            if (e.Reason == SubscriptionNotification.Remove || e.Reason == SubscriptionNotification.AttributeChange) return;

            if (e.Path.StartsWith("/ttt/image"))
            {
                if (e.Value != null)
                {
                    string name = (string)e.Value;
                    //this.LoadBitmapFromDefaultDirectory(name);
                    this.InitializeLayers(DEFAULT_DIRECTORY, name, "png");

                }
            }
            else if (e.Path.StartsWith("/ttt/material"))
            {
                if (e.Value != null)
                {
                }
            }

        }
*/

        #endregion



        #region Bitmap

        private Dictionary<string, ArrowLine> pointerArrows = new Dictionary<string, ArrowLine>();
        private Dictionary<string, ArrowLine> pointerArrowsShadow = new Dictionary<string, ArrowLine>();
        //private Dictionary<string, Ellipse> pointerArrows = new Dictionary<string, Ellipse>();


        public void RemovePointerArrow(string id)
        {
            if (this.pointerArrows.ContainsKey(id))
            {
                this.pointerArrows[id].Visibility = Visibility.Hidden;
                this.pointerArrowsShadow[id].Visibility = Visibility.Hidden;
            }
        }

        public void UpdatePointerArrow(string id, double x, double y, double startX, double startY, System.Windows.Media.Color color)
        {

            if (!this.surfaceCheckBoxShowArrows.IsChecked.Value) return;


            if (this.pointerArrows.ContainsKey(id))
            {
                this.pointerArrows[id].Visibility = Visibility.Visible;
                this.pointerArrows[id].X1 = startX;
                this.pointerArrows[id].Y1 = startY;
                this.pointerArrows[id].X2 = x;
                this.pointerArrows[id].Y2 = y;

                this.pointerArrowsShadow[id].Visibility = Visibility.Visible;
                this.pointerArrowsShadow[id].X1 = startX;
                this.pointerArrowsShadow[id].Y1 = startY;
                this.pointerArrowsShadow[id].X2 = x;
                this.pointerArrowsShadow[id].Y2 = y;
            }
            else
            {


                ArrowLine alineShadow = new ArrowLine();
                alineShadow.ArrowEnds = ArrowEnds.End;
                alineShadow.Stroke = new SolidColorBrush(Colors.White);
                alineShadow.StrokeThickness = 8;
                alineShadow.X1 = startX;
                alineShadow.Y1 = startY;
                alineShadow.X2 = x;
                alineShadow.Y2 = y;
                this.canvasPointers.Children.Add(alineShadow);
                this.pointerArrowsShadow.Add(id, alineShadow);

                // ArrowLine with animated point properties.
                ArrowLine aline = new ArrowLine();
                aline.ArrowEnds = ArrowEnds.End;
                aline.Stroke = new SolidColorBrush(color);
                aline.StrokeThickness = 4;
                aline.X1 = startX;
                aline.Y1 = startY;
                aline.X2 = x;
                aline.Y2 = y;
                this.canvasPointers.Children.Add(aline);
                this.pointerArrows.Add(id, aline);


            }

            /*
             * 
             * double radius = 8;

            if (this.pointerArrows.ContainsKey(id))
            {
                this.pointerArrows[id].SetValue(Canvas.LeftProperty, x - (radius / 2.0));
                this.pointerArrows[id].SetValue(Canvas.TopProperty, y - (radius / 2.0));
            }
            else
            {
                
                Ellipse ellipse = new Ellipse();
                ellipse.SetValue(Canvas.LeftProperty, x - (radius / 2.0));
                ellipse.SetValue(Canvas.TopProperty, y - (radius / 2.0));
                ellipse.SetValue(Canvas.WidthProperty, radius);
                ellipse.SetValue(Canvas.HeightProperty, radius);
                ellipse.Fill = new SolidColorBrush(Colors.YellowGreen);

                this.canvasPointers.Children.Add(ellipse);

                this.pointerArrows.Add(id, ellipse);
                
            }*/



        }


        public void CheckPositions()
        {
            ReadOnlyContactCollection contacts = contactTarget.GetState();
            //System.Console.WriteLine(contacts.Count);
        }


        public void InitializeLayers(string directory, string identifier, string extension)
        {
            this.Dispatcher.BeginInvoke(
                System.Windows.Threading.DispatcherPriority.Normal,
                (InitializeLayersDelegate)delegate(string directoryInt, string identifierInt, string extensionInt)
                {
                    this.InitializeLayersUnsafe(directoryInt, identifierInt, extensionInt);
                }, directory, identifier, extension);

        }

        public void InitializeLayersUnsafe(string directory, string identifier, string extension)
        {
            try
            {
                // Clear all layers
                this.layers.Clear();


                // Load the separate layers
                foreach (string layerID in this.LayerIdentifier)
                {
                    string filename = directory + identifier + "_" + layerID + "." + extension;



                    if (System.IO.File.Exists(filename))
                    {
                        if (layerID.Equals("VisibleImage"))
                        {
                            BitmapImage bitmap = new BitmapImage();
                            bitmap.BeginInit();
                            bitmap.UriSource = new Uri(filename, UriKind.RelativeOrAbsolute);
                            bitmap.EndInit();
                            this.image.Stretch = Stretch.Fill;
                            this.image.Source = bitmap;
                            

                            this.layers.Add(layerID, (System.Drawing.Bitmap)System.Drawing.Bitmap.FromFile(filename));
                        }
                        else
                        {
                            this.layers.Add(layerID, (System.Drawing.Bitmap)System.Drawing.Bitmap.FromFile(filename));
                        }

                    }
                    else
                    {
                        if (layerID.Equals(LAYER_RELIEF))
                        {
                            filename = directory + identifier + "_" + LAYER_VISIBLEIMAGE + "." + extension;

                            if (System.IO.File.Exists(filename))
                            {
                                this.layers.Add(layerID, (System.Drawing.Bitmap)System.Drawing.Bitmap.FromFile(filename));
                            }
                            else
                            {
                                this.layers.Add(layerID, new System.Drawing.Bitmap(1024, 768));
                            }
                        }
                        /*else if (layerID.Equals(LAYER_RATIO))
                        {
                            this.layers.Add(layerID, new System.Drawing.Bitmap(1024, 768));

                        }*/
                        else
                        {
                            this.layers.Add(layerID, new System.Drawing.Bitmap(1024, 768));
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.ToString());
            }
        }


        public void InitializePentimento(string directory, string identifier, string extension)
        {
            string filename = directory + identifier + "_Pentimento" + "." + extension;



            if (System.IO.File.Exists(filename))
            {
                BitmapImage bitmap = new BitmapImage();
                bitmap.BeginInit();
                bitmap.UriSource = new Uri(filename, UriKind.RelativeOrAbsolute);
                bitmap.EndInit();
                this.imagePentimento.Stretch = Stretch.Fill;
                this.imagePentimento.Source = bitmap;

            }
            this.imagePentimento.Visibility = Visibility.Visible;


        }


        public void InitializeLayersSimple(string directory, string identifier, string extension)
        {
            try
            {
                // Clear all layers
                this.layers.Clear();


                // Load the separate layers
                foreach (string layerID in this.LayerIdentifier)
                {
                    string filename = directory + identifier + "." + extension;

                    if (System.IO.File.Exists(filename))
                    {
                        if (layerID.Equals("VisibleImage"))
                        {
                            BitmapImage bitmap = new BitmapImage();
                            bitmap.BeginInit();
                            bitmap.UriSource = new Uri(filename, UriKind.RelativeOrAbsolute);
                            bitmap.EndInit();
                            this.image.Stretch = Stretch.Fill;
                            this.image.Source = bitmap;


                            this.layers.Add(layerID, (System.Drawing.Bitmap)System.Drawing.Bitmap.FromFile(filename));
                        }
                        else
                        {
                            this.layers.Add(layerID, new System.Drawing.Bitmap(1024, 768));
                        }

                    }
                }
            }
            catch (Exception ex)
            {
                System.Console.WriteLine(ex.ToString());
            }

            
        }





        private List<DependencyObject> hitResultsList = new List<DependencyObject>();

        public void ExecuteHitTest(Point pt)
        {
            // Retrieve the coordinate of the mouse position.
            //Point pt = e.GetPosition((UIElement)sender);

            // Clear the contents of the list used for hit test results.
            hitResultsList.Clear();

            // Set up a callback to receive the hit test result enumeration.
            VisualTreeHelper.HitTest(this.canvasPointers, 
                null,
                new HitTestResultCallback(MyHitTestResult),
                new PointHitTestParameters(pt));

            // Perform actions on the hit test results list.
            if (hitResultsList.Count > 0)
            {
                Console.WriteLine("Number of Visuals Hit: " + hitResultsList.Count);
            }

        }



        public HitTestResultBehavior MyHitTestResult(HitTestResult result)
        {
            // Add the hit test result to the list that will be processed after the enumeration.
            hitResultsList.Add(result.VisualHit);

            // Set the behavior to return visuals at all z-order levels.
            return HitTestResultBehavior.Continue;
        }



        public Canvas MainCanvas
        {
            get
            {
                return this.canvasPointers;
                //return this.canvasContentLayer;
            }
        }



        /// <summary>
        /// Filter the hit test values for each object in the enumeration.
        /// </summary>
        /// <param name="o"></param>
        /// <returns></returns>
        public HitTestFilterBehavior MyHitTestFilter(DependencyObject o)
        {
            // Test for the object value you want to filter.
            if (o.GetType() == typeof(Label))
            {
                // Visual object and descendants are NOT part of hit test results enumeration.
                return HitTestFilterBehavior.ContinueSkipSelfAndChildren;
            }
            else
            {
                // Visual object is part of hit test results enumeration.
                return HitTestFilterBehavior.Continue;
            }
        }




        private void ShowLayer(string layerID)
        {
            if (this.layers.ContainsKey(layerID))
            {
                System.Drawing.Bitmap bitmapBuffer = this.layers[layerID];
                //System.Drawing.Bitmap dImg = new System.Drawing.Bitmap();

                MemoryStream ms = new MemoryStream();
                bitmapBuffer.Save(ms, System.Drawing.Imaging.ImageFormat.Jpeg);
                System.Windows.Media.Imaging.BitmapImage bitmapImage = new System.Windows.Media.Imaging.BitmapImage();
                bitmapImage.BeginInit();
                bitmapImage.StreamSource = new MemoryStream(ms.ToArray());
                bitmapImage.EndInit();

                this.image.Source = bitmapImage;
            }
        }


        public bool IsShowingContacts()
        {
            return this.surfaceCheckBoxShowContacts.IsChecked.Value;
        }


        public System.Drawing.Color GetPixelColor(string layerID, Point position)
        {
            System.Drawing.Bitmap bitmapBuffer = this.layers[layerID];

            if (bitmapBuffer != null)
            {
                int x = (int)(bitmapBuffer.Width * (position.X / 1024.0));
                int y = (int)(bitmapBuffer.Height * (position.Y / 768.0));

                if (x > (bitmapBuffer.Width - 1)) x = (bitmapBuffer.Width - 1);
                else if (x < 0) x = 0;

                if (y > (bitmapBuffer.Height - 1)) y = (bitmapBuffer.Height - 1);
                else if (y < 0) y = 0;

                return bitmapBuffer.GetPixel(x,y);
            }
            else return System.Drawing.Color.Black;
        }

        #endregion



        #region Paint brush methods


        private Point CheckConstraints(Point point, double radius)
        {
            if (point.X < radius ) point.X = radius;
            if (point.Y < radius ) point.Y = radius;

            if (point.X > (1024 - radius)) point.X = (1024 - radius);
            if (point.Y > (768.0 - radius)) point.Y = (768.0 - radius);

            return point;
        }

        public void Paint(Point point, Point previousPoint, Color color, bool topLayer)
        {
            point = this.CheckConstraints(point, 8.0);
            previousPoint = this.CheckConstraints(previousPoint, 8.0);

            Line line = new Line();
            line.X1 = previousPoint.X;
            line.Y1 = previousPoint.Y;
            line.X2 = point.X;
            line.Y2 = point.Y;

            line.StrokeThickness = 15;
            line.Stroke = new SolidColorBrush(color);
            line.StrokeLineJoin = PenLineJoin.Round;

            line.StrokeStartLineCap = PenLineCap.Round;
            line.StrokeEndLineCap = PenLineCap.Round;

            

            if (topLayer)
            {
                
                this.canvasContentLayerTop.Children.Add(line);
            }
            else
            {
                this.canvasContentLayer.Children.Add(line);
            }

            /*
                Ellipse ellipse = new Ellipse();
                double radius = 30;
                // Create a SolidColorBrush with a red color to fill the 
                // Ellipse with.
                //SolidColorBrush mySolidColorBrush = new SolidColorBrush();

                // Describes the brush's color using RGB values. 
                // Each value has a range of 0-255.
                //mySolidColorBrush.Color = Color.FromArgb(255, 255, 255, 0);
                //ellipse.Fill = this.radialGradientBrush;
                ellipse.Fill = this.solidBrushGreen;

                //ellipse.StrokeThickness = 2;
                //ellipse.Stroke = Brushes.Black;

                // Set the width and height of the Ellipse.
                ellipse.Width = radius;
                ellipse.Height = radius;
                ellipse.SetValue(Canvas.LeftProperty, point.X - radius / 2.0);
                ellipse.SetValue(Canvas.TopProperty, point.Y - radius / 2.0);

                this.canvasContentLayer.Children.Add(ellipse);               
            */
        }


        #endregion




        #region Surface code: application OnClosed and handlers
        /// <summary>
        /// Occurs when the window is about to close. 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);

            // Remove handlers for Application activation events
            RemoveActivationHandlers();
        }

        /// <summary>
        /// Adds handlers for Application activation events.
        /// </summary>
        private void AddActivationHandlers()
        {
            // Subscribe to surface application activation events
            ApplicationLauncher.ApplicationActivated += OnApplicationActivated;
            ApplicationLauncher.ApplicationPreviewed += OnApplicationPreviewed;
            ApplicationLauncher.ApplicationDeactivated += OnApplicationDeactivated;
        }

        /// <summary>
        /// Removes handlers for Application activation events.
        /// </summary>
        private void RemoveActivationHandlers()
        {
            // Unsubscribe from surface application activation events
            ApplicationLauncher.ApplicationActivated -= OnApplicationActivated;
            ApplicationLauncher.ApplicationPreviewed -= OnApplicationPreviewed;
            ApplicationLauncher.ApplicationDeactivated -= OnApplicationDeactivated;
        }

        /// <summary>
        /// This is called when application has been activated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationActivated(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// This is called when application is in preview mode.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationPreviewed(object sender, EventArgs e)
        {
        }

        /// <summary>
        ///  This is called when application has been deactivated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnApplicationDeactivated(object sender, EventArgs e)
        {
        }

        private void surfaceCheckBoxWidgets_Checked(object sender, RoutedEventArgs e)
        {
            this.canvasContentLayer.Visibility = Visibility.Visible;
        }

        private void surfaceCheckBoxWidgets_Unchecked(object sender, RoutedEventArgs e)
        {
            this.canvasContentLayer.Visibility = Visibility.Hidden;
        }


        private void surfaceCheckShowLayers_Checked(object sender, RoutedEventArgs e)
        {
            this.surfaceListBoxLayers.Visibility = Visibility.Visible;
        }

        private void surfaceCheckShowLayers_Unchecked(object sender, RoutedEventArgs e)
        {
            this.surfaceListBoxLayers.Visibility = Visibility.Hidden;
        }

        private void surfaceListBoxLayers_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                ListBoxItem item = (ListBoxItem)e.AddedItems[0];

                string id = item.Content.ToString();
                //System.Console.WriteLine("Selected item: " + id);

                this.ShowLayer(id);
            }
        }

        private void surfaceCheckBoxShowContacts_Checked(object sender, RoutedEventArgs e)
        {

        }

        #endregion




        #region Pentimento


        int pentimentoRadius = 120;

        Dictionary<string, PentimentiMarker> pentimentiMarker = new Dictionary<string, PentimentiMarker>();
        DrawingGroup pentimentiDrawingGroup = new DrawingGroup();


        private void InitPentimento()
        {
            GeometryDrawing box =
                new GeometryDrawing(
                    new SolidColorBrush(Color.FromArgb(0, 0, 0, 0)),
                    null,
                    new RectangleGeometry(new Rect(0.0, 0.0, 1024.0, 768.0))
                );
            this.pentimentiDrawingGroup.Children.Add(box);

            DrawingBrush drawingBrush = new DrawingBrush();
            //GeometryDrawing geometryDrawing = new GeometryDrawing();

            this.imagePentimento.OpacityMask = drawingBrush;
            

            drawingBrush.Drawing = this.pentimentiDrawingGroup;

            //drawingBrush.Drawing = geometryDrawing;
            //geometryDrawing.Geometry = this.pentimentiDrawingGroup;
        }


        public void ShowPentimento(string artefactID, Point position, double intensity)
        {
            if (this.pentimentiMarker.ContainsKey(artefactID))
            {
                this.pentimentiMarker[artefactID].Intensity = intensity;
                this.pentimentiMarker[artefactID].Position = position;

                if (position.X < pentimentoRadius / 2.0) position.X = pentimentoRadius / 2.0;
                if (position.Y < pentimentoRadius / 2.0) position.Y = pentimentoRadius / 2.0;

                if (position.X > (1024 - pentimentoRadius / 2.0)) position.X = (1024 - pentimentoRadius / 2.0);
                if (position.Y > (768.0 - pentimentoRadius / 2.0)) position.Y = (768.0 - pentimentoRadius / 2.0);


                this.pentimentiMarker[artefactID].EllipseGeometry.Center = position;

                this.pentimentiMarker[artefactID].GradientStop.Color = Color.FromArgb((byte)(255 * intensity), 0, 0, 0);
                this.pentimentiMarker[artefactID].GradientStop2.Color = Color.FromArgb((byte)(255 * intensity), 0, 0, 0);
            }
            else
            {
                PentimentiMarker marker = new PentimentiMarker();
                this.pentimentiMarker.Add(artefactID, marker);
                marker.Intensity = intensity;
                marker.Position = position;
                RadialGradientBrush brush = new RadialGradientBrush();
                marker.Brush = brush;
                //RadialGradientBrush brush = new RadialGradientBrush();
                marker.GradientStop2 = new GradientStop(Color.FromArgb(0, 0, 0, 0), 1);
                brush.GradientStops.Add(marker.GradientStop2);
                marker.GradientStop = new GradientStop(Color.FromArgb(0, 0, 0, 0), 0);
                brush.GradientStops.Add(marker.GradientStop);

                //<RadialGradientBrush x:Name="brushPentimento">
                //    <GradientStop Color="#00000000" Offset="1" />
                //    <GradientStop Color="#00000000" Offset="0" x:Name="gradientStopPentimento" />
                //</RadialGradientBrush>


                marker.EllipseGeometry = new EllipseGeometry(position, this.pentimentoRadius,this.pentimentoRadius);

                GeometryDrawing ellipse =
                    new GeometryDrawing(
                        marker.Brush,
                        new Pen(Brushes.Transparent, 0),
                        marker.EllipseGeometry
                    );
                this.pentimentiDrawingGroup.Children.Add(ellipse);

            }


            /*DrawingBrush drawingBrush = new DrawingBrush();
            this.imagePentimento.OpacityMask = new DrawingBrush();

            this.imagePentimento.OpacityMask = drawingBrush;
            GeometryDrawing geometryDrawing = new GeometryDrawing();
            GeometryGroup group = new GeometryGroup();

            drawingBrush.Drawing = geometryDrawing;
            geometryDrawing.Geometry = group;
            */


            // Create a DrawingGroup to contain the drawings.


            /*
             * this.brushPentimento.MappingMode = BrushMappingMode.Absolute;
            this.brushPentimento.RadiusX = this.pentimentoRadius;
            this.brushPentimento.RadiusY = this.pentimentoRadius;

            this.brushPentimento.GradientOrigin = position;
            this.brushPentimento.Center = position;

            this.gradientStopPentimento.Color = Color.FromArgb((byte)(255 * intensity), 0, 0, 0);
            */
        }




        #endregion




        #region Controls

        private void surfaceCheckBoxToggleControls_Checked(object sender, RoutedEventArgs e)
        {
            if (this.canvasControls != null)
            {
                this.canvasControls.Visibility = Visibility.Visible;
            }
        }

        private void surfaceCheckBoxToggleControls_Unchecked(object sender, RoutedEventArgs e)
        {
            this.canvasControls.Visibility = Visibility.Hidden;
        }

        private void surfaceButtonHideControls_Click(object sender, RoutedEventArgs e)
        {
            this.surfaceCheckBoxToggleControls.IsChecked = false;
        }

        private void surfaceListBoxFiles_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, this.surfaceListBoxFiles.SelectedItem.ToString(), "png");

        }

        #endregion


        #region Demonstration Preset Code

        private void surfaceRadioButton1_Checked(object sender, RoutedEventArgs e)
        {
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
            this.InitializeLayers(DEFAULT_DIRECTORY, "Photo 1", "png");
            if (this.surfaceListBoxFiles != null)
            {
                this.surfaceListBoxFiles.IsEnabled = true;
            }
        }

        private void surfaceRadioButton1_Unchecked(object sender, RoutedEventArgs e)
        {
            if (this.surfaceListBoxFiles != null)
            {
                this.surfaceListBoxFiles.IsEnabled = false;
            }
        }

        private void surfaceRadioButton2_Checked(object sender, RoutedEventArgs e)
        {
            //this.InitializeLayersSimple(DEFAULT_DIRECTORY, "White", "png");

            this.SetDemonstrationPreset(DemonstrationPreset.LayerPainting);
            //this.canvasPointers.Background = null;
            //this.image.Visibility = Visibility.Hidden;
            this.canvasContentLayer.Visibility = Visibility.Visible;
            this.canvasContentLayerTop.Visibility = Visibility.Visible;

            this.InitializeLayers(DEFAULT_DIRECTORY, "Mona_Lisa", "png");
        }

        private void surfaceRadioButton2_Unchecked(object sender, RoutedEventArgs e)
        {
            //this.image.Visibility = Visibility.Visible;
            this.canvasContentLayer.Visibility = Visibility.Hidden;
            this.canvasContentLayerTop.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton3_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "Night Cafe", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Art);
            this.InitializeContainers(DemonstrationPreset.Art);
        }

        private void surfaceRadioButton4_Checked(object sender, RoutedEventArgs e)
        {
            this.SetDemonstrationPreset(DemonstrationPreset.Baskets);
            this.InitializeLayers(DEFAULT_DIRECTORY, "Sand 1", "png");
            this.InitializeContainers(DemonstrationPreset.Baskets);

            this.boxInner.Visibility = Visibility.Visible;
            this.boxOuter.Visibility = Visibility.Visible;
            this.imageBasket.Visibility = Visibility.Visible;
            this.ellipse1.Visibility = Visibility.Visible;
            this.ellipse2.Visibility = Visibility.Visible;
            this.ellipse3.Visibility = Visibility.Visible;
            this.ellipse4.Visibility = Visibility.Visible;
        }
        
        private void surfaceRadioButton4_Unchecked(object sender, RoutedEventArgs e)
        {
            this.boxInner.Visibility = Visibility.Hidden;
            this.boxOuter.Visibility = Visibility.Hidden;
            this.imageBasket.Visibility = Visibility.Hidden;
            this.ellipse1.Visibility = Visibility.Hidden;
            this.ellipse2.Visibility = Visibility.Hidden;
            this.ellipse3.Visibility = Visibility.Hidden;
            this.ellipse4.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton5_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayersSimple(DEFAULT_DIRECTORY, "Screen1", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Payment);
            this.InitializeContainers(DemonstrationPreset.Payment);
        }

        private void surfaceRadioButton6_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayersSimple(DEFAULT_DIRECTORY, "Floorplan 4", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.FloorPlanning);
            this.InitializeContainers(DemonstrationPreset.FloorPlanning);

            this.imageFurnitureConference.Visibility = Visibility.Visible;
            //this.imageFurnitureGroup.Visibility = Visibility.Visible;
            this.imageFurnitureConference2.Visibility = Visibility.Visible;
            this.imageFurnitureGroup2.Visibility = Visibility.Visible;
            this.imageFurnitureTable.Visibility = Visibility.Visible;
            this.image1.Visibility = Visibility.Visible;
            //this.image2.Visibility = Visibility.Visible;
            //this.image3.Visibility = Visibility.Visible;
            this.image4.Visibility = Visibility.Visible;
            this.image5.Visibility = Visibility.Visible;
            this.image6.Visibility = Visibility.Visible;
            //this.image7.Visibility = Visibility.Visible;
            this.image8.Visibility = Visibility.Visible;
        }

        private void surfaceRadioButton6_Unchecked(object sender, RoutedEventArgs e)
        {
            this.imageFurnitureConference.Visibility = Visibility.Hidden;
            this.imageFurnitureGroup.Visibility = Visibility.Hidden;
            this.imageFurnitureConference2.Visibility = Visibility.Hidden;
            this.imageFurnitureGroup2.Visibility = Visibility.Hidden;
            this.imageFurnitureTable.Visibility = Visibility.Hidden;

            this.image1.Visibility = Visibility.Hidden;
            this.image2.Visibility = Visibility.Hidden;
            //this.image3.Visibility = Visibility.Hidden;
            this.image4.Visibility = Visibility.Hidden;
            this.image5.Visibility = Visibility.Hidden;
            this.image6.Visibility = Visibility.Hidden;
            //this.image7.Visibility = Visibility.Hidden;
            this.image8.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton7_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayersSimple(DEFAULT_DIRECTORY, "Black", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.RockerSwitch);
        }


        

        private void surfaceRadioButton8_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "mapa-mundi", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton9_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "Oscillation", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton10_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "Materials", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton11_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "Borders Force Feedback", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton12_Checked(object sender, RoutedEventArgs e)
        {
            this.InitializeLayers(DEFAULT_DIRECTORY, "Island", "png");
            this.imageCloud.Visibility = Visibility.Visible;
            this.SetDemonstrationPreset(DemonstrationPreset.Baskets);
        }

        private void surfaceRadioButton12_Unchecked(object sender, RoutedEventArgs e)
        {
            this.imageCloud.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton13_Checked(object sender, RoutedEventArgs e)
        {
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
            this.InitializeLayers(DEFAULT_DIRECTORY, "Terrain", "png");
            /*if (this.surfaceListBoxFiles != null)
            {
                this.surfaceListBoxFiles.IsEnabled = true;
            }*/
        }


        

        private void surfaceButtonClearPaintingLayers_Click(object sender, RoutedEventArgs e)
        {
            this.canvasContentLayer.Children.Clear();
            this.canvasContentLayerTop.Children.Clear();
        }

        private void surfaceRadioButton14_Checked(object sender, RoutedEventArgs e)
        {
            // Goya
            this.InitializeLayers(DEFAULT_DIRECTORY, "Goya Painting", "png");
            this.InitializePentimento(DEFAULT_DIRECTORY, "Goya Painting", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Pentimento);
        }

        private void surfaceRadioButton14_Unchecked(object sender, RoutedEventArgs e)
        {
            // Goya end
            this.imagePentimento.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton15_Checked(object sender, RoutedEventArgs e)
        {
            // Picasso
            this.InitializeLayers(DEFAULT_DIRECTORY, "Picasso - old guitarist", "png");
            this.InitializePentimento(DEFAULT_DIRECTORY, "Picasso - old guitarist", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Pentimento);
        }

        private void surfaceRadioButton15_Unchecked(object sender, RoutedEventArgs e)
        {
            // Picasso end
            this.imagePentimento.Visibility = Visibility.Hidden;
        }

        private void surfaceRadioButton16_Checked(object sender, RoutedEventArgs e)
        {
            // Africa and india
            this.InitializeLayers(DEFAULT_DIRECTORY, "Africa and India", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton17_Checked(object sender, RoutedEventArgs e)
        {
            // Luna
            this.InitializeLayers(DEFAULT_DIRECTORY, "Luna", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);

        }

        
        

        private void surfaceRadioButton7_Checked_1(object sender, RoutedEventArgs e)
        {
            // Stone relief
            this.InitializeLayers(DEFAULT_DIRECTORY, "Stone Image", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }
    
        

        private void surfaceRadioButton18_Checked(object sender, RoutedEventArgs e)
        {
            // Water video
            this.InitializeLayers(DEFAULT_DIRECTORY, "Water 2", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);



            //if (storyboard != null && mediaElement != null)
           // {
              //  this.storyboard.Begin();
                //this.mediaElement.Play();
                this.mediaElement.Visibility = Visibility.Visible;
           // }
        }

        private void surfaceRadioButton19_Checked(object sender, RoutedEventArgs e)
        {
            // XRAY
            this.InitializeLayers(DEFAULT_DIRECTORY, "xray", "png");
            this.SetDemonstrationPreset(DemonstrationPreset.Default);
        }

        private void surfaceRadioButton18_Unchecked(object sender, RoutedEventArgs e)
        {
            //if (storyboard != null && mediaElement != null)
            //{
                // Unload water video
               // this.storyboard.Stop();
                //this.timeline
                //this.mediaElement.Stop();
                this.mediaElement.Visibility = Visibility.Hidden;
            //}
        }

        #endregion

        internal void NotifyActivity(string notifyID)
        {
            foreach (string id in this.tttArtefacts.Keys)
            {
                if (id != notifyID)
                {
                    this.tttArtefacts[id].NotifyActivity();
                }
            }
        }




        // Hide and show pointer arrows
        private void surfaceCheckBoxShowArrows_Checked(object sender, RoutedEventArgs e)
        {
            foreach (ArrowLine al in this.pointerArrows.Values)
            {
                al.Visibility = Visibility.Visible;
            }
            foreach (ArrowLine al in this.pointerArrowsShadow.Values)
            {
                al.Visibility = Visibility.Visible;
            }
        }

        private void surfaceCheckBoxShowArrows_Unchecked(object sender, RoutedEventArgs e)
        {
            foreach(ArrowLine al in this.pointerArrows.Values)
            {
                al.Visibility = Visibility.Hidden;
            }
            foreach (ArrowLine al in this.pointerArrowsShadow.Values)
            {
                al.Visibility = Visibility.Hidden;
            }
        }
    }







    public enum DemonstrationPreset
    {
        Default,
        LayerPainting,
        Art,
        Baskets,
        Payment,
        FloorPlanning,
        RockerSwitch,
        Pentimento
    }

    public class PentimentiMarker
    {
        public Point Position;
        public double Intensity;
        public Ellipse Ellipse;
        public Brush Brush;
        public EllipseGeometry EllipseGeometry;
        public GradientStop GradientStop;
        public GradientStop GradientStop2;
    }

    
}